% tmseeg_fillGaps() - Fill the gap from the TMS data removal by applying a
% weighted forward and backward prediction. It utilizes and other packet
% functions
%
%Inputs:  A        - parent GUI information (structure)
%         step_num - Step number for current cleaning step in workflow
%  
%Display window Information:
%   Check [button] - To check that the selected duration of the prediction
%   segments is within data range
%   Ok [button] - Starts process to fill gap
%   Cancel [button] - Aborts step

function tmseeg_fillGaps(A, step_num)

%Check if previous steps were done
if tmseeg_previous_step(step_num)
    return 
end
global backcolor VARS

% Execute gap filling?
fillGaps = questdlg('Fill time periods deleted with estimated data?');
if strcmp(fillGaps,'No')
    [files, EEG] = tmseeg_load_step(step_num);
    tmseeg_step_check(files, EEG, A, step_num);
else
    [files,EEG] = tmseeg_load_step(step_num);
    % check if completed in previous step
    if EEG.pnts == EEG.epoch_length*1000
        warndlg('Prediction has already been applied. Exiting...','Move to next step');
        tmseeg_step_check(files, EEG, A, step_num);
        return;
    end
%--------------------- GUI DISPLAY ---------------------    
    S.chfig = figure('Menubar','none',...
        'Toolbar','none',...
        'Name','tmseeg_fillGaps',...
        'Numbertitle','off',...
        'Resize','off',...
        'Color',backcolor,...
        'DockControls','off');
    % initial variables
    time_samples = strcat(num2str(VARS.FRONT_DUR),',',num2str(VARS.BACK_DUR));
    memb = num2str(VARS.DIMENS);
    nnei = num2str(VARS.NEIGH);
    
    S.title_txt  = uicontrol('Parent',S.chfig,'Style','text',...
        'String','Set Variables for Data Prediction and Time Series Reconstruction',...
        'fontweight','bold',...
        'Units','normalized',...
        'Position',[0.1 0.9 0.8 0.05]); 
    S.dur_txt = uicontrol('Parent', S.chfig,'Style', 'text',...
        'String', 'Duration of datasets prior and after TMS event (in ms):',...
        'Units','normalized',...
        'Position',[0.05 0.7 0.6 0.05]); 
    S.dur_note = uicontrol('Parent', S.chfig,'Style', 'text',...
        'String', 'Default duration is that of the full epoch',...
        'fontAngle','italic',...
        'Units','normalized',...
        'Position',[0.05 0.65 0.455 0.05]);
    S.dur_note2 = uicontrol('Parent', S.chfig,'Style', 'text',...
        'String', 'In both cases you can input only two values',...
        'fontAngle','italic',...
        'Units','normalized',...
        'Position',[0.05 0.6 0.5 0.05]);
    S.check_button = uicontrol('Parent',S.chfig, 'Style', 'pushbutton',...
        'String','check',...
        'Units','normalized',...
        'Position',[0.85 0.65 0.1 0.05],...
        'Callback',{@check_limits_button, S, EEG});
    S.dur_time = uicontrol('Parent',S.chfig, 'Style', 'edit',...
        'String',time_samples,...
        'Units','normalized',...
        'Position',[0.75 0.7 0.2 0.05]);
    S.dimens_txt= uicontrol('Parent',S.chfig,'Style', 'text',...
        'String', 'Number of components for the devision of the time series:',...
        'Units','normalized',...
        'Position',[0.05 0.5 0.58 0.05]); 
    S.dim_val = uicontrol('Parent',S.chfig, 'Style', 'edit',...
        'String',memb,...
        'Units','normalized',...
        'Position',[0.75 0.5 0.2 0.05]);
    S.nei_txt = uicontrol('Parent',S.chfig, 'Style', 'text',...
        'String', 'Number of Nearset Neighbors to search for:',...
        'Units','Normalized',...
        'Position',[0.05 0.3 0.49 0.05]);
    S.nei_val = uicontrol('Parent',S.chfig, 'Style', 'edit',...
        'String',nnei,...
        'Units','normalized',...
        'Position',[0.75 0.3 0.2 0.05]);
    S.ok_button = uicontrol('Parent',S.chfig, 'Style', 'pushbutton',...
        'String','ok',...
        'Units','normalized',...
        'Position',[0.75 0.1 0.1 0.05],...
        'BackgroundColor',[1 1 1],...
        'Callback',{@start_fillGaps,S,A,step_num,EEG,files});
    S.cancel_button2 = uicontrol('Parent',S.chfig, 'Style', 'pushbutton',...
        'String','cancel',...
        'Units','normalized',...
        'Position',[0.85 0.1 0.1 0.05],...
        'BackgroundColor',[1 1 1],...
        'Callback',{@cancel_call, S});
    guidata(S.chfig,S);
end
end

% check that the duration of the prediction segments does not exceed data
% range
function check_limits_button(varargin)
S = varargin{3};
S = guidata(S.chfig);
EEG = varargin{4};
%add gap
EpochSecs = EEG.epoch_length;
EEG = tmseeg_addTMSTimeBack(EEG, EpochSecs);

%recover value for durations-variables
time_samples = str2num(char(get(S.dur_time,'String')));
sbeforerec = time_samples(1,1)*EEG.srate/1000;
safterrec = time_samples(1,2)*EEG.srate/1000;

if ~exist('EEG.period2remove_b')
    %single pulse
    if -sbeforerec < EEG.times(1,1) || safterrec > EEG.times(1,end)
        error('Time duration of learning sets are out of data range. Note that time value is in ms.');
    else
        msgbox('Time Durations Check out!');
    end
else
    %Paired pulse - If the durations are left as initialized:
    %For the first pulse :  The duration of the forward prediction segment
    %is set form the beginning of the epoch until the beginning of the gap.
    %The duration of the backward prediction is set from the end of the
    %first gap until the beginning of second gap.
    %For the second pulse : The duration of the forward prediction segment
    %is set as the interval between the two gaps and the backward
    %prediction segment is set from the end of the second gap to the end of
    %the epoch.
    
    imin1 = EEG.event(1).latency; %index of the first TMS event
    imin2 = EEG.event(1).latency; %index of the second TMS event
    
    %check in comparison to EEG data range
    if -sbeforerec < EEG.times(1,1) || safterrec > EEG.times(1,end)
        error('Time duration of learning sets are out of data range. Note that time value is in ms.');
    end
    
    %check for gap interlap and data range
    if -sbeforerec == EEG.times(1,1)
        if safterrec == EEG.times(1,end)
            msgbox('Time Durations Check out!');
        else
            if EEG.times(imin1)+safterrec> EEG.times(1,EEG.TMS_period2remove_b(1,1)) || EEG.times(imin2) + safterrec > EEG.times(1,end)
                error('Exceeds limits and overlaps with TMS data!');
            else
                 msgbox('Time Durations Check out!');
            end
        end
    elseif safterrec == EEG.times(1,end)
        if -sbeforerec == EEG.times(1,1)
            msgbox('Time Durations Check out!');
        else
            if  -EEG.times(imin2)+sbeforerec < EEG.times(1,EEG.TMS_period2remove_1(1,end)) || EEG.times(imin1) - sbeforerec< EEG.times(1,1) 
                error('Exceeds limits and overlaps with TMS data!');    
            else
                msgbox('Time Durations Check out!');
            end
        end
    elseif EEG.times(imin1) - sbeforerec< EEG.times(1,1) || EEG.times(imin2) + safterrec > EEG.times(1,end) 
        error('Exceeds limits!');
    elseif EEG.times(imin2)-sbeforerec < EEG.times(1,EEG.period2remove_1(1,end)) || EEG.times(imin1)+safterrec> EEG.times(1,EEG.TMS_period2remove_b(1,1))
        error('Overlaps with TMS data!');
    end  
end    
end

function start_fillGaps(varargin)
S = varargin{3};
A = varargin{4};
step_num = varargin{5};
EEG = varargin{6};
files = varargin{7};

time_samples = str2num(char(get(S.dur_time,'String')));
memb = str2num(char(get(S.dim_val,'String')));
nnei = str2num(char(get(S.nei_val,'String')));

VARS.FRONT_DUR = time_samples(1);
VARS.BACK_DUR = time_samples(2);
VARS.NEIGH = nnei;
VARS.DIMENS = memb;

guidata(S.chfig,S);
close

EpochSecs = EEG.epoch_length;
EEG = tmseeg_addTMSTimeBack(EEG, EpochSecs);
%small correction in struct fields 
EEG.xmax = -EEG.xmin-0.001;
EEG.times = EEG.times(1:end-1);

if time_samples(1,2) == EEG.times(1,end)+1
    time_samples(1,2) = time_samples(1,2)-1;
end
%retrieve duration of prediction segments
sbeforerec = time_samples(1,1)*EEG.srate/1000;
safterrec = time_samples(1,2)*EEG.srate/1000;

%secondary check
if -sbeforerec < EEG.times(1,1) || safterrec > EEG.times(1,end)
    error('Time duration of learning sets are out of data range. Note that time value is in ms.');
end

if ~exist('EEG.TMS_period2remove_b')
    %Single Pulse 
    
    %retrieve indexes denoting the start, end of gap and the TMS event 
    sbeforeTMS = EEG.TMS_period2remove_1(1,1);
    safterTMS = EEG.TMS_period2remove_1(1,end);
    imin = EEG.event(1).latency;
    
    %execute extra baseline correction (for the backward prediction)
    base = questdlg('Correct baseline?');
    if strcmp(base,'Yes')
        %estimated the duration for the basline correction
        dur = EEG.pnts/4;
        npreTMS = dur*EEG.srate/1000;
        npostTMS = npreTMS;
        %basline correct
        EEG = baselineCorForGaps(EEG,npreTMS,npostTMS,sbeforeTMS,safterTMS,imin);
    end
    %fill gaps
    EEG = fillGaps(EEG,sbeforeTMS,safterTMS,sbeforerec,safterrec,imin,memb,nnei);
else
    %Double Pulse   
    %retrieve the indexes corresponding to the start and end of each gap
    sbeforeTMS1 = EEG.TMS_period2remove_1(1,1);
    safterTMS1 = EEG.TMS_period2remove_1(1,end);
    sbeforeTMS2 = EEG.TMS_period2remove_b(1,1) + length(EEG.TMS_period2remove_1);
    safterTMS2 = EEG.TMS_period2remove_b(1,end) + length(EEG.TMS_period2remove_1);
    
    %TMS event indexes of the two TMS events
    imin1 = fix(EEG.event(1).latency);
    imin2 = fix(EEG.event(2).latency);
    
    %set the actual durations of the prediction segemtns for each pulse
    if -sbeforerec == EEG.times(1,1)
        sbeforerec1 = abs(EEG.times(1,1)) - abs(EEG.times(1,imin1));
        sbeforerec2 = abs(EEG.times(1,safterTMS1))+EEG.times(1,imin2);
    else
        sbeforerec1 = sebeforerec;
        sbeforerec2 = sbeforerec;
    end
    if safterrec == EEG.times(1,end)
        safterrec1 = abs(EEG.times(1,imin1))+EEG.times(1,EEG.TMS_period2remove_b(1,1));
        safterrec2 = abs(EEG.times(1,end)) - abs(EEG.times(1,imin2))-1;
    else
        safterrec1 = safterrec;
        safterrec2 = safterrec;
    end
    %secondary check
    if EEG.times(imin1) - sbeforerec1< EEG.times(1,1) || EEG.times(imin2) + safterrec2 > EEG.times(1,end)
        error('Exceeds limits and overlaps with TMS data!');
    elseif -EEG.times(imin2)+sbeforerec2 < EEG.times(1,EEG.TMS_period2remove_1(1,end)) || EEG.times(imin1)+safterrec1> EEG.times(1,EEG.TMS_period2remove_b(1,1))
        error('Exceeds limits and overlaps with TMS data!');
    end
    %execute extra baseline correction (for the backward prediction)
	base = questdlg('Correct baseline?');
    if strcmp(base,'Yes')
        %estimate the appropriate duration for the baseline correction
        %segments
        dur = EEG.pnts/8;
        dur = min([safterrec1, safterrec2,sbeforerec1,sbeforerec2, dur]);
        npreTMS = dur*EEG.srate/1000;
        npostTMS = npreTMS;
        %execute correction for each pulse
        EEG = baselineCorForGaps(EEG,npreTMS,npostTMS,sbeforeTMS1,safterTMS1,imin1);
        EEG = baselineCorForGaps(EEG,npreTMS,npostTMS,sbeforeTMS2,safterTMS2,imin2);
    end
    %fill gaps
	EEG = fillGaps(EEG,sbeforeTMS1,safterTMS1,sbeforerec1,safterrec1,imin1,memb,nnei);
    EEG = fillGaps(EEG,sbeforeTMS2,safterTMS2,sbeforerec2,safterrec2,imin2,memb,nnei);

end
%save file
tmseeg_step_check(files, EEG, A, step_num);
msgbox('Done!')
end

function cancel_call(varargin)
%abort step
S = varargin{3};
guidata(S.chfig,S);
close
return;
end
